SEESPO ; #######################################################
 ; SIMPLE KEYBOARD GRAPHICS PATCH REPLACES NORMAL CONTROL
 ; CODES PRODUCED BY THE SHIFT-DOWN ARROW AND MAKES USE OF
 ; THE CURRENTLY UNDEFINED SHIFT-UP ARROW.  GRAPHICS CHAR-
 ; ACTERS ARE DISPLAYED ON THE VIDEO SCREEN.  NOTE THAT
 ; EDITING OF STRINGS AND LINES CONTAINING GRAPHICS IS NOT
 ; POSSIBLE, AND THE LINE MUST BE COMPLETELY RE-ENTERED.
 ; ALSO, IF THIS ROUTINE IS TO BE USED WITH OTHER ROUTINES
 ; SUCH AS KEYBOARD OR UPPER/LOWER CASE DRIVERS, THE PATCH
 ; INTO THE KEYBOARD SCAN MUST STORE THE PREVIOUS JUMP
 ; ADDRESS AT 4016, AND USE IT AS ITS JUMP ADDRESS AT THE
 ; END OF THE PROGRAM.  SOME KEYS DUPLICATE GRAPHICS.
 ; #######################################################
 ;
 	ORG	7C00H		; CHANGE TO RELOCATE
 ;
 ; #######################################################
 ; THIS BLOCK PATCHES IN ROUTINE'S ADDRESS.  CHANGE THIS
 ; PATCH METHOD IF U/L DRIVERS, ETC., ARE TO BE USED.
 ; #######################################################
 ;
 START	LD	HL,(4016H)	; KEYBOARD PATCH POINT
 	LD	(JUMPER),HL	; RETURN POINT SETUP
 	LD	HL,ENTRY	; GET ENTRY POINT READY
 	LD	(4016H),HL	; ENTRY POINT TO PATCH
 	JP	06CCH		; RETURN TO BASIC
 ;
 ; #######################################################
 ; BLOCK BELOW VERIFIES THE PRESENCE OF AN ARROW KEY
 ; #######################################################
 ;
 ENTRY	LD	BC,3840H	; KEYBOARD ROW ADDRESS
 	LD	A,(BC)		; GET KEYBOARD DATA
 	AND	18H		; CHECK IF UP/DN ARROW
 	JR	Z,AWAY		; BEGONE IF NOT
 ;
 ; #######################################################
 ; BLOCK BELOW VERIFIES THE PRESENCE OF THE SHIFT KEY
 ; #######################################################
 ;
 	RLC	C		; GET SHIFT KEY ADDRESS
 	LD	A,(BC)		; GET SHIFT KEY DATA
 	CP	01		; SEE IF SHIFT PRESSED
 	JR	NZ,AWAY		; BEGONE IF NOT
 ;
 ; #######################################################
 ; BLOCK BELOW CHECKS KEYBOARD TO DISCOVER DEPRESSED KEYS
 ; #######################################################
 ;
 	LD	HL,4036H	; GET KEYBOARD BUFFER
 	RLC	C		; GET FIRST KEYBOARD ROW
 	LD	D,0		; SET DATA COUNTER
 AGAIN	LD	A,(BC)		; CHECK KEYBOARD DATA
 	LD	E,A		; SAVE DATA TO FREE A
 	AND	E		; CHECK TO SEE IF ZERO
 	JR	NZ,FOUND	; GO IF KEY DEPRESSED
 	LD	(HL),A		; CLEAR KEY ROW BUFFER
 	INC	D		; INCREMENT DATA COUNT
 	INC	L		; INCREMENT BUFFER ADD.
 	RLC	C		; GET NEXT KEYBOARD ROW
 	LD	A,C		; PUT ROW COUNTER IN A
 	SUB	40H		; SUB. FINAL ROW VALUE
 	JR	NZ,AGAIN	; GO BACK IF VALID ROW
 	JR	ENTRY		; TRY AGAIN IF LAST ROW
 FOUND	AND	(HL)		; CHECK IF SAME KEY
 	JR	NZ,ENTRY	; TRY AGAIN IF SAME KEY
 	LD	(HL),E		; SAVE VALUE IN BUFFER
 ;
 ; #######################################################
 ; BLOCK BELOW IS A STANDARD DEBOUNCE ROUTINE.  IF THIS IS
 ; USED WITH OTHER DRIVERS, ELIMINATE THIS BLOCK, OR THE
 ; DEBOUNCE WILL BE EXCESSIVE RESULTING IN LOST CHARACTERS
 ; #######################################################
 ;
 	PUSH	BC		; SAVE BC VALUES
 	LD	BC,200H		; LOAD DELAY VALUE
 	CALL	0060H		; CALL DELAY SUBROUTINE
 	POP	BC		; RESTORE BC VALUES
 	LD	A,(BC)		; AGAIN CHECK KEYBOARD
 	AND	E		; AND WITH VALUE SAVED
 	JR	Z,ENTRY		; RELOOP IF A BOUNCE
 ;
 ; #######################################################
 ; BLOCK BELOW TAKES ROW AND CONVERTS IT TO COUNTER VALUE
 ; #######################################################
 ;
 	LD	A,D		; GET DATA COUNTER BACK
 	RLCA			; MOVE IT OVER
 	RLCA			; MOVE IT OVER
 	RLCA			; AND ONE MORE TIME
 	LD	D,A		; SAVE PLAYTIME VALUE
 ;
 ; #######################################################
 ; BLOCK BELOW CONVERTS KEY DATA TO UNIQUE WORKABLE VALUE
 ; #######################################################
 ;
 	LD	C,1		; SET UP SECOND COUNTER
 NOTYET	LD	A,C		; VALUE READY FOR PLAY
 	AND	E		; CHECK IF A MATCH
 	JR	NZ,GOTIT	; YEA, GO IF A MATCH
 	INC	D		; INCRMT. SHIFTED DATA
 	RLC	C		; READY NEXT MATCH TEST
 	JR	NOTYET		; GO BACK AND TRY AGAIN
 ;
 ; #######################################################
 ; BLOCK BELOW CREATES GRAPHICS FROM THE FINAL VALUE FOUND
 ; #######################################################
 ;
 GOTIT	LD	A,(3840H)	; GET ARROW ROW AGAIN
 	CP	10H		; SEE IF DOWN ARROW
 	JR	Z,NOBIT		; GET GOING IF IT IS
 	LD	A,90H		; UP ARROW SHIFT VALUE
 	JR	THERE		; SKITTER PAST MATH
 NOBIT	LD	A,80H		; DOWN ARROW SHIFT 
 THERE	ADD	A,D		; CREATE FULL GRAPHICS
 ;
 ; #######################################################
 ; JUMP/RETURN BLOCK - CHANGE THIS TO JUMP TO OTHER DRIVER
 ; #######################################################
 ;
 	RET			; BACK AND DISPLAY IT
 AWAY	DEFB	0C3H		; DEFINES JUMP COMMAND
 JUMPER	DEFB	0E3H		; DEFINES NORMAL LSB
 	DEFB	003H		; DEFINES NORMAL MSB
 ;
 ; #######################################################
 	END	START		; DEFINE ENTRY ADDRESS
